<?php

namespace Drupal\trackmyadmin\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\trackmyadmin\Utility\QueryHelper;
use Drupal\Component\Utility\Xss;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\File\FileSystemInterface;

/**
 * Reports Controller.
 */
class Reports extends ControllerBase {

  /**
   * The replica database connection.
   *
   * @var \Drupal\Database
   */
  protected $connection;

  /**
   * The QueryHelper Object.
   *
   * @var Drupal\trackmyadmin\Utility\QueryHelper
   */
  protected $query;

  /**
   * Constructs the Reports Object.
   */
  public function __construct() {
    $this->connection = \Drupal::database();
    $this->query = new QueryHelper($this->connection);
  }

  /**
   * Returns a render-able array for the Dashboard page.
   */
  public function dashboard() {
    $data = $this->query->fetchValues('content_config');
    $data['base_path'] = \Drupal::request()->getSchemeAndHttpHost();

    return [
      // Your theme hook name.
      '#theme' => 'trackmyadmin_reports_theme',
      // Your variables.
      '#data' => $data,
    ];
  }

  /**
   * Returns a render-able array for the Name Report page.
   *
   * @param string $name
   *   The Identifier Name.
   */
  public function name($name) {
    $data = $this->query->fetchValues('name', ['data' => Xss::filter($name)]);
    $data['base_path'] = \Drupal::request()->getSchemeAndHttpHost();
    $data['current_data'] = 'name';
    $data['data_item'] = Xss::filter($name);

    return [
      // Your theme hook name.
      '#theme' => 'trackmyadmin_reports_data_theme',
      // Your variables.
      '#data' => $data,
    ];
  }

  /**
   * Returns a render-able array for the User Report page.
   *
   * @param string $user
   *   The Identifier User.
   */
  public function user($user) {
    $user_details = user_load_by_name(Xss::filter($user));
    $data = $this->query->fetchValues('uid', ['data' => $user_details->id()]);
    $data['base_path'] = \Drupal::request()->getSchemeAndHttpHost();
    $data['current_data'] = 'user';
    $data['data_item'] = Xss::filter($user);

    return [
      // Your theme hook name.
      '#theme' => 'trackmyadmin_reports_data_theme',
      // Your variables.
      '#data' => $data,
    ];
  }

  /**
   * Returns a render-able array for the Team Report page.
   *
   * @param string $team
   *   The Identifier Team.
   */
  public function team($team) {
    $data = $this->query->fetchValues('team', ['data' => Xss::filter($team)]);
    $data['base_path'] = \Drupal::request()->getSchemeAndHttpHost();
    $data['current_data'] = 'team';
    $data['data_item'] = Xss::filter($team);

    return [
      // Your theme hook name.
      '#theme' => 'trackmyadmin_reports_data_theme',
      // Your variables.
      '#data' => $data,
    ];
  }

  /**
   * Returns a render-able array for the Content Report page.
   */
  public function tmacontent() {
    $data = $this->query->fetchValues('content');
    $data['base_path'] = \Drupal::request()->getSchemeAndHttpHost();
    $data['current_data'] = 'Content';

    return [
      // Your theme hook name.
      '#theme' => 'trackmyadmin_reports_data_view_theme',
      // Your variables.
      '#data' => $data,
    ];
  }

  /**
   * Returns a render-able array for the Config Report page.
   */
  public function tmaconfig() {
    $data = $this->query->fetchValues('config');
    $data['base_path'] = \Drupal::request()->getSchemeAndHttpHost();
    $data['current_data'] = 'Config';

    return [
      // Your theme hook name.
      '#theme' => 'trackmyadmin_reports_data_view_theme',
      // Your variables.
      '#data' => $data,
    ];
  }

  /**
   * Returns a render-able array for the Data Flush Report page.
   */
  public function tmaflush() {
    $data = $this->query->fetchValues('flush_data');
    $data['base_path'] = \Drupal::request()->getSchemeAndHttpHost();
    $data['current_data'] = 'Data Flush';

    return [
      // Your theme hook name.
      '#theme' => 'trackmyadmin_reports_data_view_theme',
      // Your variables.
      '#data' => $data,
    ];
  }

  /**
   * Exports the data in CSV Format.
   */
  public function export() {
    $file_path = $this->download();
    \Drupal::messenger()->addStatus(t('Download the CSV File <a href=":url">Here</a>.', [':url' => $file_path]));
    $response = new RedirectResponse('/admin/reports/trackmyadmin/settings/export');
    return $response->send();
  }

  /**
   * Flush the data in Dashboard.
   */
  public function flush() {
    // Let us make sure we download a copy before we flush the data.
    $file_path = $this->download();
    // Initiate the flush.
    $delete_key = $this->query->deleteValues('trackmyadmin_activity');
    $session = \Drupal::request()->getSession();
    $tma_user_key = $session->get('tma_user_key');
    $values = [
      'id' => $tma_user_key,
      'field' => 'Data Flush',
      'value' => 'Data Flush',
      'url' => \Drupal::service('path.current')->getPath(),
      'modified_type' => 'flush_data',
      'date' => \Drupal::time()->getCurrentTime()
    ];
    // Make an entry of the data flush happened.
    $inserted_key = $this->query->insertValues('trackmyadmin_activity', $values);
    \Drupal::messenger()->addStatus(t('Data Flush Successful. Download the CSV File <a href=":url">Here</a>.', [':url' => $file_path]));
    $response = new RedirectResponse('/admin/reports/trackmyadmin/settings/flush');
    return $response->send();
  }

  /**
   * Helper function to download the CSV file.
   *
   * @return string
   *   Downloaded path of the CSV file.
   */
  public function download() {
    $logged_in_date = $date = $file_path = NULL;
    $download_folder = 'public://trackmyadmin';
    // Create the directory if needed.
    if (\Drupal::service('file_system')->prepareDirectory($download_folder, FileSystemInterface::CREATE_DIRECTORY)) {
      $file_name = 'TMAData_' . date('m-d-Y_H', time()) . '.csv';
      $file_path = file_create_url('public://trackmyadmin/' . $file_name);
      $fileObject = fopen($download_folder . '/' . $file_name, 'a');
      $data = $this->query->fetchValues('entire_data');
      if (!empty($data)) {
        $header_field = ['User Identified As',
          'From Team',
          'Logged in As',
          'Logged in On',
          'From Location',
          'Using IP',
          'Modified Field Name',
          'Modified Field Value',
          'Modified On Page',
          'Modified Type',
          'Modified On'
        ];
        // Insert the CSV Table headers.
        fputcsv($fileObject, $header_field);
        foreach ($data as $data_records) {
          $user = User::load($data_records->uid);
          if (is_numeric($data_records->logged_in_date)) {
            $logged_in_date = \Drupal::service('date.formatter')->format($data_records->logged_in_date, 'custom', 'm-d-Y H:i:s');
          }
          if (is_numeric($data_records->date)) {
            $date = \Drupal::service('date.formatter')->format($data_records->date, 'custom', 'm-d-Y H:i:s');
          }
          $fields = [$data_records->name,
            $data_records->team,
            $user->getAccountName(),
            $logged_in_date,
            $data_records->location,
            $data_records->ip,
            $data_records->field,
            $data_records->value,
            $data_records->url,
            $data_records->modified_type,
            $date
          ];
          // The rest of the fields.
          fputcsv($fileObject, $fields);
        }
      }
    }
    return $file_path;
  }

}
